@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2019   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,10 +42,11 @@
  * Current state of client for a given group
  */
 enum client_phase {
-    PHASE_REGISTERED = 1,       /// Registered and awaiting KEYINFO or REG_CONF
-    PHASE_RECEIVING = 2,        /// Currently receiving a file
-    PHASE_COMPLETE = 3,         /// Completed group and awaiting DONE_CONF
-    PHASE_MIDGROUP = 4          /// Registered awaiting next file or group end
+    PHASE_PREREGISTER = 1,      /// 
+    PHASE_REGISTERED = 2,       /// Registered and awaiting KEYINFO or REG_CONF
+    PHASE_RECEIVING = 3,        /// Currently receiving a file
+    PHASE_COMPLETE = 4,         /// Completed group and awaiting DONE_CONF
+    PHASE_MIDGROUP = 5          /// Registered awaiting next file or group end
 };
 
 /**
@@ -58,7 +59,7 @@ struct file_t {
     uint32_t secsize_small, secsize_big;  /// Size of sections
     int ftype;                  /// File type (regular, directory, symlink)
     f_offset_t size;            /// Size in bytes
-    int32_t tstamp;             /// File timestamp
+    int64_t tstamp;             /// File timestamp
     char filepath[MAXPATHNAME]; /// Local path to file
     char temppath[MAXPATHNAME]; /// Local path to temp file
     char name[MAXPATHNAME];     /// Path name
@@ -133,7 +134,10 @@ struct group_list_t {
     char start_time[10];            /// Time initial ANNOUNCE was received
     uint16_t send_seq;              /// Outgoing seq. number for loss detection
     uint32_t src_id;                /// ID of server (network byte order)
-    union sockaddr_u replyaddr;
+    uint32_t proxy_id;              /// ID of proxy (network byte order)
+    struct fp_list_t *server_fp;    /// Pointer to server's fingerprint struct
+    struct fp_list_t *proxy_fp;     /// Pointer to proxy's fingerprint struct
+    union sockaddr_u replyaddr;     /// IP to send responses to
     int phase;                      /// Current client_phase of the group
     int client_auth, restart, sync_mode, sync_preview; /// Flags from ANNOUNCE
     struct client_restart_t *restartinfo; /// Restart file header
@@ -153,27 +157,39 @@ struct group_list_t {
     uint32_t gsize;                 /// Group size estimate
     struct timeval cc_time;         /// Timer for sending CC_ACK
     struct timeval last_server_ts, last_server_rx_ts;
-    int keytype, hashtype, sigtype, keyextype;   /// Encryption parameters
+    int keytype, hashtype;          /// Encryption parameters
     union key_t server_pubkey;      /// Server's public key
-    union key_t client_privkey;     /// Client's private key for this group
     union key_t server_dhkey;       /// Server ECDH public key for this group
-    union key_t client_dhkey;       /// Client ECDH public key for this group
-    unsigned int server_pubkeylen;  /// Length in bytes of server key
-    unsigned int client_privkeylen; /// Length in bytes of client key
+    union key_t proxy_pubkey;       /// Response proxy public key
+    union key_t proxy_dhkey;        /// Respose proxy ECDH key for this group
+    union key_t client_dhkey;       /// Client ECDH private key for this group
+    unsigned int server_keytype;    /// Type of server key
+    unsigned int proxy_keytype;     /// Type of response proxy key
     uint8_t rand1[RAND_LEN];        /// Server's random number
     uint8_t rand2[RAND_LEN];        /// Client's random number
-    uint8_t premaster[MASTER_LEN];  /// Premaster secret sent by client
+    uint8_t *s_context;             /// Server session context
+    uint8_t *p_context;             /// Proxy session context
+    uint8_t *c_context1;            /// Client session context 1
+    uint8_t *c_context2;            /// Client session context 2
+    int s_context_len;              /// Length of server context
+    int p_context_len;              /// Length of proxy context
+    int c_context1_len;             /// Length of client context 1
+    int c_context2_len;             /// Length of client context 2
+    uint8_t premaster[MASTER4_LEN]; /// Premaster secret resulting from ECDH
+    uint8_t groupmaster[MASTER4_LEN];/// Group master key from server
     unsigned int premaster_len;     /// Length of premaster secret
-    uint8_t master[MASTER_LEN];     /// Master key for client
-    uint8_t hmackey[HMAC_LEN];      /// HMAC key for client
-    uint8_t key[MAXKEY];            /// Symmetric encryption key for client
-    uint8_t salt[MAXIV];            /// Salt for block cypher IV for client
-    uint8_t groupmaster[MASTER_LEN];/// Master key for server
-    uint8_t grouphmackey[HMAC_LEN]; /// HMAC key for server
-    uint8_t groupkey[MAXKEY];       /// Symmetric encryption key for server
-    uint8_t groupsalt[MAXIV];       /// Salt for block cypher IV for server
+    uint8_t s_hs_key[MAXKEY];       /// Symmetric handshake key for server
+    uint8_t s_hs_iv[MAXIV];         /// Symmetric handshake IV for server
+    uint8_t c_hs_key[MAXKEY];       /// Symmetric handshake key for client
+    uint8_t c_hs_iv[MAXIV];         /// Symmetric handshake IV for client
+    uint8_t s_app_key[MAXKEY];      /// Symmetric application key for server
+    uint8_t s_app_iv[MAXIV];        /// Symmetric application IV for server
+    uint8_t c_app_key[MAXKEY];      /// Symmetric application key for client
+    uint8_t c_app_iv[MAXIV];        /// Symmetric application IV for client
+    uint8_t finished_key[HASH_LEN]; /// Key for client finished message
+    uint8_t verify_data[HASH_LEN];  /// Client's finished hash
     uint64_t ivctr;                 /// Counter portion of the IV
-    int ivlen, keylen, hmaclen;     /// Length of HMAC key, symmetric key and iv
+    int ivlen, keylen, hashlen;     /// Length of hash, symmetric key and iv
     struct file_t fileinfo;         /// Info pertaining to current file
 };
 
@@ -183,29 +199,29 @@ struct group_list_t {
 extern SOCKET listener;
 extern char tempdir[MAXDIRNAME], destdir[MAXDIR][MAXDIRNAME];
 extern char pidfile[MAXPATHNAME];
-extern char keyfile[MAXLIST][MAXPATHNAME], keyinfo[MAXLIST][MAXPATHNAME];
+extern char keyfile[MAXPATHNAME], keyinfo[MAXPATHNAME];
 extern char backupdir[MAXDIR][MAXDIRNAME];
 extern char statusfilename[MAXPATHNAME];
 extern FILE *status_file;
-extern int debug, encrypted_only, dscp, destdircnt, tempfile, keyinfo_count;
-extern int interface_count, pub_multi_count, keyfile_count, rcvbuf, backupcnt;
+extern int debug, encrypted_only, dscp, destdircnt, tempfile;
+extern int interface_count, pub_multi_count, rcvbuf, backupcnt;
 extern char postreceive[MAXPATHNAME], portname[PORTNAME_LEN];
-extern int port, move_individual, cache_len, noname;
+extern int port, move_individual, cache_len, noname, user_abort, use_ssm;
 extern uint32_t uid;
 extern union sockaddr_u hb_hosts[MAXLIST];
 extern struct iflist m_interface[MAX_INTERFACES];
 extern union sockaddr_u pub_multi[MAX_INTERFACES];
 extern struct group_list_t group_list[MAXLIST];
-extern struct fp_list_t server_keys[MAXLIST];
+extern struct fp_list_t server_list[MAXLIST], proxy_list[MAXLIST];
 extern struct iflist ifl[MAX_INTERFACES];
 extern struct timeval next_keyreq_time, next_hb_time;
-extern int ifl_len, server_count, key_count, has_proxy, sys_keys, priority;
+extern int ifl_len, server_count, proxy_count, has_v4_proxy, sys_keys, priority;
 extern int hbhost_count, hb_interval;
-extern union key_t privkey[MAXLIST];
-extern int privkey_type[MAXLIST];
-extern struct fp_list_t proxy_info;
-extern union key_t proxy_pubkey, proxy_dhkey;
-extern int proxy_pubkeytype;
+extern union key_t privkey;
+extern int privkey_type;
+extern struct fp_list_t proxy4_info;
+extern union key_t proxy4_pubkey, proxy4_dhkey;
+extern int proxy4_pubkeytype;
 
 #endif  // _CLIENT_H
 
