@@ -334,7 +334,7 @@ void send_register(struct group_list_t *group)
     } else {
         glog2(group, "REGISTER sent");
     }
-    glog3(group, "send time: %d.%06d", send_time.tv_sec, send_time.tv_usec);
+    glog3(group, "send time: %ld.%06ld", send_time.tv_sec, send_time.tv_usec);
 
     set_timeout(group, 0);
     if (group->client_auth) {
@@ -480,7 +480,7 @@ void send_fileinfo_ack(struct group_list_t *group, int restart)
     } else {
         glog2(group, "FILEINFO_ACK sent");
     }
-    glog3(group, "send time: %d.%06d", send_time.tv_sec, send_time.tv_usec);
+    glog3(group, "send time: %ld.%06ld", send_time.tv_sec, send_time.tv_usec);
     free(encrypted);
     free(buf);
 }
