@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2022   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -138,7 +138,7 @@ void read_restart_file(struct group_list_t *group)
     }
     if (rval != sizeof(struct client_restart_t)) {
         glog0(group, "Failed to read header for restart file "
-                "(read %d, expected %d)", rval,sizeof(struct client_restart_t));
+                "(read %d, expected %zu)",rval,sizeof(struct client_restart_t));
         goto err1;
     }
 
@@ -285,7 +285,7 @@ void run_postreceive_multi(struct group_list_t *group, char *const *files,
         for (i = 1; i < count + 3; i++) {
             sclog2(" %s", params[i]);
         }
-        slog2("");
+        slog2(" ");
     }
 
 #ifdef WINDOWS
@@ -540,7 +540,8 @@ int flush_disk_cache(struct group_list_t *group)
         group->fileinfo.curr_offset = offset + wrote_len;
         if (wrote_len != group->fileinfo.cache_len) {
             glog0(group, "Write failed for blocks %d - %d, only wrote %d bytes",
-                        group->fileinfo.cache_start, group->fileinfo.cache_end);
+                        group->fileinfo.cache_start, group->fileinfo.cache_end,
+                        wrote_len);
             return 0;
         } else {
             glog4(group, "Wrote blocks %d - %d to disk from cache",
