@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2019   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,8 +47,8 @@ void send_abort(struct group_list_t *group, const char *message);
 void handle_abort(struct group_list_t *group, const unsigned char *message,
                   unsigned meslen);
 void send_key_req(void);
-void handle_proxy_key(const union sockaddr_u *src,
-                      unsigned char *message, unsigned meslen);
+void handle_v4_proxy_key(const union sockaddr_u *src,
+                         unsigned char *message, unsigned meslen);
 void clear_path(const char *path, struct group_list_t *group);
 void move_to_backup(struct group_list_t *group);
 int create_path_to_file(struct group_list_t *group, const char *filename);
