@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2019   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,46 +55,52 @@
 SOCKET listener;
 char tempdir[MAXDIRNAME], destdir[MAXDIR][MAXDIRNAME];
 char pidfile[MAXPATHNAME];
-char keyfile[MAXLIST][MAXPATHNAME], keyinfo[MAXLIST][MAXPATHNAME];
+char keyfile[MAXPATHNAME], keyinfo[MAXPATHNAME];
 char backupdir[MAXDIR][MAXDIRNAME];
 char statusfilename[MAXPATHNAME];
 FILE *status_file;
-int debug, encrypted_only, dscp, destdircnt, tempfile, keyinfo_count;
-int interface_count, pub_multi_count, keyfile_count, rcvbuf, backupcnt;
+int debug, encrypted_only, dscp, destdircnt, tempfile;
+int interface_count, pub_multi_count, rcvbuf, backupcnt;
 char postreceive[MAXPATHNAME], portname[PORTNAME_LEN];
-int port, move_individual, cache_len, noname;
+int port, move_individual, cache_len, noname, user_abort, use_ssm;
 uint32_t uid;
 union sockaddr_u hb_hosts[MAXLIST];
 struct iflist m_interface[MAX_INTERFACES];
 union sockaddr_u pub_multi[MAX_INTERFACES];
 struct group_list_t group_list[MAXLIST];
-struct fp_list_t server_keys[MAXLIST];
+struct fp_list_t server_list[MAXLIST], proxy_list[MAXLIST];
 struct iflist ifl[MAX_INTERFACES];
 struct timeval next_keyreq_time, next_hb_time;
-int ifl_len, server_count, key_count, has_proxy, sys_keys, priority;
+int ifl_len, server_count, proxy_count, has_v4_proxy, sys_keys, priority;
 int hbhost_count, hb_interval;
-union key_t privkey[MAXLIST];
-int privkey_type[MAXLIST];
-struct fp_list_t proxy_info;
-union key_t proxy_pubkey, proxy_dhkey;
-int proxy_pubkeytype;
+union key_t privkey;
+int privkey_type;
+struct fp_list_t proxy4_info;
+union key_t proxy4_pubkey, proxy4_dhkey;
+int proxy4_pubkeytype;
 
 extern char *optarg;
 extern int optind;
 
 /**
- * Adds a server and its fingerprint to the list of approved servers
+ * Adds a server/proxy and its fingerprint to the appropriate list
  */
-void add_server_by_name(const char *server, const char *ip,
-                        const char *fingerprint)
+void add_host_to_list(struct fp_list_t *list, int *list_count,
+                      const char *uid_str, const char *ip,
+                      const char *proxyid_str, const char *fingerprint)
 {
     struct addrinfo ai_hints, *ai_rval;
-    uint32_t server_uid;
+    uint32_t host_uid, proxy_uid;
     int rval;
 
-    server_uid = strtoul(server, NULL, 16);
-    if ((server_uid == 0xffffffff) || (server_uid == 0)) {
-        fprintf(stderr, "Invalid server UID %s\n", server);
+    host_uid = strtoul(uid_str, NULL, 16);
+    if ((host_uid == 0xffffffff) || (host_uid == 0)) {
+        fprintf(stderr, "Invalid host UID %s\n", uid_str);
+        exit(ERR_PARAM);
+    }
+    proxy_uid = strtoul(proxyid_str, NULL, 16);
+    if (proxy_uid == 0xffffffff) {
+        fprintf(stderr, "Invalid proxy UID %s\n", proxyid_str);
         exit(ERR_PARAM);
     }
 
@@ -104,18 +110,19 @@ void add_server_by_name(const char *server, const char *ip,
     ai_hints.ai_protocol = 0;
     ai_hints.ai_flags = 0;
     if ((rval = getaddrinfo(ip, NULL, &ai_hints, &ai_rval)) != 0) {
-        fprintf(stderr, "Invalid server name/address %s: %s\n",
+        fprintf(stderr, "Invalid host name/address %s: %s\n",
                 ip, gai_strerror(rval));
         exit(ERR_PARAM);
     }
 
-    server_keys[server_count].uid = htonl(server_uid);
-    memcpy(&server_keys[server_count].addr, ai_rval->ai_addr,
+    list[*list_count].uid = htonl(host_uid);
+    list[*list_count].proxy_uid = htonl(proxy_uid);
+    memcpy(&list[*list_count].addr, ai_rval->ai_addr,
             ai_rval->ai_addrlen);
-    server_keys[server_count].has_fingerprint =
-            parse_fingerprint(server_keys[server_count].fingerprint,
+    list[*list_count].has_fingerprint =
+            parse_fingerprint(list[*list_count].fingerprint,
                               fingerprint);
-    server_count++;
+    (*list_count)++;
     freeaddrinfo(ai_rval);
 }
 
@@ -146,12 +153,11 @@ void set_defaults(void)
     destdircnt = 0;
     backupcnt = 0;
     pub_multi_count = 0;
-    key_count = 0;
-    keyfile_count = 0;
-    keyinfo_count = 0;
+    memset(keyfile, 0, sizeof(keyfile));
+    memset(keyinfo, 0, sizeof(keyinfo));
     rcvbuf = 0;
     server_count = 0;
-    has_proxy = 0;
+    has_v4_proxy = 0;
     sys_keys = 0;
     memset(hb_hosts, 0, sizeof(hb_hosts));
     hbhost_count = 0;
@@ -162,7 +168,9 @@ void set_defaults(void)
     max_log_size = 0;
     max_log_count = DEF_MAX_LOG_COUNT;
     cache_len = DEF_CACHE;
-    proxy_pubkeytype = 0;
+    proxy4_pubkeytype = 0;
+    user_abort = 0;
+    use_ssm = 0;
 }
 
 /**
@@ -173,10 +181,11 @@ void process_args(int argc, char *argv[])
     int c, i, listidx, rval;
     long tmpval;
     struct addrinfo ai_hints, *ai_rval;
-    char line[1000], *servername, *ipstr, *fingerprint;
-    char *p, *p2, *hoststr, *portstr, pubname[INET6_ADDRSTRLEN];
-    FILE *serverfile;
-    const char opts[]="dx:qF:L:P:s:c:I:p:tT:D:A:M:B:Q:EU:S:R:k:K:mN:ig:n:h:H:";
+    char line[1000], *serverid, *proxyid, *ipstr, *fingerprint;
+    char *p, *p2, *saveptr, *hoststr, *portstr, pubname[INET6_ADDRSTRLEN];
+    FILE *serverfile, *proxyfile;
+    const char opts[] =
+            "dx:qF:L:P:s:c:I:p:tT:D:A:M:B:Q:EU:oS:R:r:k:K:mN:ig:n:h:H:";
 
     set_defaults();
 
@@ -330,6 +339,9 @@ void process_args(int argc, char *argv[])
             }
             uid = htonl(uid);
             break;
+        case 'o':
+            use_ssm = 1;
+            break;
         case 'S':
             if ((serverfile = fopen(optarg, "r")) == NULL) {
                 fprintf(stderr, "Couldn't open server list %s: %s\n",
@@ -337,31 +349,24 @@ void process_args(int argc, char *argv[])
                 exit(ERR_PARAM);
             }
             while (fgets(line, sizeof(line), serverfile)) {
-                while ((strlen(line) != 0) && ((line[strlen(line)-1] == '\r') ||
-                       (line[strlen(line)-1] == '\n'))) {
-                    line[strlen(line)-1] = '\x0';
-                }
                 if ((line[0] == '#') || (line[0] == '\x0')) {
                     continue;
                 }
-                servername = line;
-                ipstr = strchr(servername, '|');
-                if (ipstr) {
-                    *ipstr = '\x0';
-                    ipstr++;
-                    fingerprint = strchr(ipstr, '|');
-                    if (fingerprint) {
-                        *fingerprint = '\x0';
-                        fingerprint++;
-                    }
-                } else {
-                    fingerprint = NULL;
-                }
-                if (strlen(servername) >= DESTNAME_LEN) {
-                    fprintf(stderr, "Server list: name too long\n");
+                saveptr = NULL;
+                serverid = strtok_r(line, "|", &saveptr);
+                ipstr = strtok_r(NULL, "|", &saveptr);
+                if (!ipstr) {
+                    fprintf(stderr, "Missing IP in server config");
                     exit(ERR_PARAM);
                 }
-                add_server_by_name(servername, ipstr, fingerprint);
+                proxyid = strtok_r(NULL, "|\r\n", &saveptr);
+                if (!proxyid) {
+                    fprintf(stderr, "Missing proxy ID in server config");
+                    exit(ERR_PARAM);
+                }
+                fingerprint = strtok_r(NULL, "|\r\n", &saveptr);
+                add_host_to_list(server_list, &server_count, serverid, ipstr,
+                                 proxyid, fingerprint);
             }
             if (!feof(serverfile) && ferror(serverfile)) {
                 perror("Failed to read from server list file");
@@ -370,10 +375,37 @@ void process_args(int argc, char *argv[])
             fclose(serverfile);
             break;
         case 'R':
+            if ((proxyfile = fopen(optarg, "r")) == NULL) {
+                fprintf(stderr, "Couldn't open proxy list %s: %s\n",
+                        optarg, strerror(errno));
+                exit(ERR_PARAM);
+            }
+            while (fgets(line, sizeof(line), proxyfile)) {
+                if ((line[0] == '#') || (line[0] == '\x0')) {
+                    continue;
+                }
+                saveptr = NULL;
+                proxyid = strtok_r(line, "|", &saveptr);
+                ipstr = strtok_r(NULL, "|\r\n", &saveptr);
+                if (!ipstr) {
+                    fprintf(stderr, "Missing IP in proxy config");
+                    exit(ERR_PARAM);
+                }
+                fingerprint = strtok_r(NULL, "|\r\n", &saveptr);
+                add_host_to_list(proxy_list, &proxy_count, proxyid, ipstr,
+                                 "0", fingerprint);
+            }
+            if (!feof(proxyfile) && ferror(proxyfile)) {
+                perror("Failed to read from proxy list file");
+                exit(ERR_PARAM);
+            }
+            fclose(proxyfile);
+            break;
+        case 'r':
             strncpy(line, optarg, sizeof(line));
             line[sizeof(line)-1] = '\x0';
-            servername = strtok(line, "/");
-            if (!servername) {
+            ipstr = strtok(line, "/");
+            if (!ipstr) {
                 fprintf(stderr, "Invalid host name\n");
                 exit(ERR_PARAM);
             }
@@ -383,35 +415,25 @@ void process_args(int argc, char *argv[])
             ai_hints.ai_socktype = SOCK_DGRAM;
             ai_hints.ai_protocol = 0;
             ai_hints.ai_flags = 0;
-            if ((rval = getaddrinfo(servername, NULL,
+            if ((rval = getaddrinfo(ipstr, NULL,
                     &ai_hints, &ai_rval)) != 0) {
                 fprintf(stderr, "Invalid proxy address %s: %s\n",
-                        servername, gai_strerror(rval));
+                        ipstr, gai_strerror(rval));
                 exit(ERR_PARAM);
             }
-            memcpy(&proxy_info.addr, ai_rval->ai_addr, ai_rval->ai_addrlen);
-            proxy_info.has_fingerprint =
-                    parse_fingerprint(proxy_info.fingerprint, fingerprint);
-            has_proxy = 1;
+            memcpy(&proxy4_info.addr, ai_rval->ai_addr, ai_rval->ai_addrlen);
+            proxy4_info.has_fingerprint =
+                    parse_fingerprint(proxy4_info.fingerprint, fingerprint);
+            has_v4_proxy = 1;
             freeaddrinfo(ai_rval);
             break;
         case 'k':
-            p = strtok(optarg, ",");
-            while (p != NULL) {
-                strncpy(keyfile[keyfile_count], p, sizeof(keyfile[0])-1);
-                keyfile[keyfile_count][sizeof(keyfile[0])-1] = '\x0';
-                keyfile_count++;
-                p = strtok(NULL, ",");
-            }
+            strncpy(keyfile, optarg, sizeof(keyfile)-1);
+            keyfile[sizeof(keyfile)-1] = '\x0';
             break;
         case 'K':
-            p = strtok(optarg, ",");
-            while (p != NULL) {
-                strncpy(keyinfo[keyinfo_count], p, sizeof(keyinfo[0])-1);
-                keyinfo[keyinfo_count][sizeof(keyinfo[0])-1] = '\x0';
-                keyinfo_count++;
-                p = strtok(NULL, ",");
-            }
+            strncpy(keyinfo, optarg, sizeof(keyinfo)-1);
+            keyinfo[sizeof(keyinfo)-1] = '\x0';
             break;
         case 'm':
             sys_keys = 1;
@@ -494,7 +516,11 @@ void process_args(int argc, char *argv[])
             exit(ERR_PARAM);
         }
     }
-    if (server_count) {
+    if (use_ssm) {
+        if (!server_count) {
+            fprintf(stderr, "SSM mode enabled but no server list specified\n");
+            exit(ERR_PARAM);
+        }
         for (i = 0; i < pub_multi_count; i++) {
             if (!is_multicast(&pub_multi[i], 1)) {
                 if ((rval = getnameinfo((struct sockaddr *)&pub_multi[i],
@@ -512,17 +538,25 @@ void process_args(int argc, char *argv[])
                     "for source specific multicast\n", DEF_PUB_MULTI);
             exit(ERR_PARAM);
         }
+        // for SSM, make sure server entry with a proxy has a proxy entry
+        for (i = 0; i < server_count; i++) {
+            if (server_list[i].proxy_uid) {
+                if (!fp_lookup(server_list[i].proxy_uid, proxy_list,
+                               proxy_count)) {
+                    fprintf(stderr, "Proxy %08x for server %08x"
+                            " not in proxy list\n",
+                            ntohl(server_list[i].proxy_uid),
+                            ntohl(server_list[i].uid));
+                    exit(ERR_PARAM);
+                }
+            }
+        }
     }
-    if ((keyfile_count != 0) && (keyinfo_count != 0) &&
-            (keyfile_count != keyinfo_count)) {
-        fprintf(stderr, "Must list same number of items for -k and -K\n");
-        exit(ERR_PARAM);
-    }
-    if (has_proxy) {
-        if (proxy_info.addr.ss.ss_family == AF_INET6) {
-            proxy_info.addr.sin6.sin6_port = htons(port);
+    if (has_v4_proxy) {
+        if (proxy4_info.addr.ss.ss_family == AF_INET6) {
+            proxy4_info.addr.sin6.sin6_port = htons(port);
         } else {
-            proxy_info.addr.sin.sin_port = htons(port);
+            proxy4_info.addr.sin.sin_port = htons(port);
         }
     }
     if (destdircnt == 0) {
