@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2022   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -536,8 +536,8 @@ void handle_fileinfo(struct group_list_t *group, const unsigned char *message,
     if (tempfile && !group->sync_preview) {
         clear_path(group->fileinfo.temppath, group);
     }
-    if ((group->fileinfo.ftype != FTYPE_DELETE) ||
-            (group->fileinfo.ftype != FTYPE_FREESPACE)) {
+    if ((group->fileinfo.ftype == FTYPE_DELETE) ||
+            (group->fileinfo.ftype == FTYPE_FREESPACE)) {
         // Don't do path checks for metafile commands
     } else if (lstat_func(group->fileinfo.filepath, &statbuf) != -1) {
         glog3(group, "checking existing file");
