@@ -107,7 +107,7 @@ struct timeval *getrecenttimeout(void)
                 } else if ((!found_timeout) ||
                            (cmptimestamp(group->timeout_time,
                                          min_timestamp) < 0)) {
-                    glog5(group, "found min timeout time: %d:%06d",
+                    glog5(group, "found min timeout time: %ld:%06ld",
                                  group->timeout_time.tv_sec,
                                  group->timeout_time.tv_usec);
                     min_timestamp = group->timeout_time;
@@ -159,7 +159,7 @@ struct timeval *getrecenttimeout(void)
                            ((!found_timeout) ||
                             (cmptimestamp(group->fileinfo.nak_time,
                                           min_timestamp) < 0))) {
-                    glog5(group, "found min nak time: %d:%06d",
+                    glog5(group, "found min nak time: %ld:%06ld",
                          group->fileinfo.nak_time.tv_sec,
                          group->fileinfo.nak_time.tv_usec);
                     min_timestamp = group->fileinfo.nak_time;
@@ -175,7 +175,7 @@ struct timeval *getrecenttimeout(void)
                                ((!found_timeout) ||
                                 (cmptimestamp(group->cc_time,
                                               min_timestamp) < 0))) {
-                        glog5(group, "found min CC time: %d:%06d",
+                        glog5(group, "found min CC time: %ld:%06ld",
                              group->cc_time.tv_sec, group->cc_time.tv_usec);
                         min_timestamp = group->cc_time;
                         found_timeout = 1;
@@ -262,7 +262,7 @@ void mainloop(void)
         }
         tv = getrecenttimeout();
         if (tv) {
-            log5(0, 0, 0, "read timeout: %d.%06d", tv->tv_sec, tv->tv_usec);
+            log5(0, 0, 0, "read timeout: %ld.%06ld", tv->tv_sec, tv->tv_usec);
         }
         if (read_packet(listener, &src, buf, &packetlen,
                         MAXMTU, tv, &tos) <= 0) {
