@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2023   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -603,6 +603,7 @@ void send_cc_ack(struct group_list_t *group)
     group->cc_time.tv_sec = 0;
     group->cc_time.tv_usec = 0;
     free(buf);
+    free(encrypted);
 }
 
 /**
@@ -620,7 +621,7 @@ void init_tfmcc_fb_round(struct group_list_t *group, uint16_t new_ccseq)
     gettimeofday(&group->cc_time, NULL);
     add_timeval_d(&group->cc_time, (backoff > 0) ? backoff : 0);
     group->initrate = current_cc_rate(group);
-    glog3(group, "Starting feedback round %d: backoff = %.3f, initrate = %d",
+    glog3(group,"Starting feedback round %u: backoff = %.3f, initrate = " F_i64,
                  new_ccseq, backoff, group->initrate);
 }
 
@@ -887,9 +888,9 @@ unsigned int get_naks(struct group_list_t *group,
     if (log_level >= 5) {
         for (i = 0; i < group->blocksize; i++) {
             sclog5("%02X ", (*naks)[i]);
-            if (i % 25 == 24) slog5("");
+            if (i % 25 == 24) slog5(" ");
         }
-        slog5("");
+        slog5(" ");
     }
 
     if (numnaks == 0) {
@@ -1060,6 +1061,7 @@ void handle_cong_ctrl(struct group_list_t *group, const unsigned char *message,
     } else {
         time_us = (int64_t)ntohl(cong_ctrl->tstamp_hi) << 32;
         time_us |= ntohl(cong_ctrl->tstamp_lo);
+        group->last_server_ts = usec_to_tv(time_us);
     }
     group->last_server_rx_ts = rxtime;
 
