@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2015   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,8 +58,6 @@ static const struct keyinfo_t {
     int keysize;
     int blocksize;
 } keyinfo[] = {
-    { KEY_DES, 8, 8 },
-    { KEY_DES_EDE3, 24, 8 },
     { KEY_AES128_CBC, 16, 16 },
     { KEY_AES128_GCM, 16, 12 },
     { KEY_AES128_CCM, 16, 12 },
@@ -224,10 +222,6 @@ static BCRYPT_ALG_HANDLE get_alg_handle(LPCWSTR alg, LPCWSTR mode, int hmac)
 static LPCWSTR get_cipher(int keytype)
 {
     switch (keytype) {
-    case KEY_DES:
-        return BCRYPT_DES_ALGORITHM;
-    case KEY_DES_EDE3:
-        return BCRYPT_3DES_ALGORITHM;
     case KEY_AES128_CBC:
     case KEY_AES256_CBC:
     case KEY_AES128_GCM:
@@ -247,8 +241,6 @@ static LPCWSTR get_cipher(int keytype)
 static LPCWSTR get_cipher_mode(int keytype)
 {
     switch (keytype) {
-    case KEY_DES:
-    case KEY_DES_EDE3:
     case KEY_AES128_CBC:
     case KEY_AES256_CBC:
         return BCRYPT_CHAIN_MODE_CBC;
@@ -277,8 +269,6 @@ static LPCWSTR get_hash(int hashtype)
         return BCRYPT_SHA256_ALGORITHM;
     case HASH_SHA1:
         return BCRYPT_SHA1_ALGORITHM;
-    case HASH_MD5:
-        return BCRYPT_MD5_ALGORITHM;
     default:
         log0(0, 0, 0, "Unknown hashtype: %d", hashtype);
         return NULL;
@@ -381,7 +371,7 @@ int cipher_supported(int keytype)
 int hash_supported(int hashtype)
 {
     LPCWSTR alg;
-    if ((alg = get_hash(hashtype)) == NULL) {
+    if ((hashtype == HASH_SHA1) || ((alg = get_hash(hashtype)) == NULL)) {
         return 0;
     }
     return alg_found(alg, BCRYPT_HASH_OPERATION);
@@ -869,10 +859,10 @@ int create_RSA_sig(RSA_key_t rsa, int hashtype,
  */
 int verify_RSA_sig(RSA_key_t rsa, int hashtype,
                    const unsigned char *mes, unsigned int meslen,
-                   unsigned char *sig, unsigned int siglen)
+                   const unsigned char *sig, unsigned int siglen)
 {
     SECURITY_STATUS sstatus;
-    unsigned char meshash[HMAC_LEN];
+    unsigned char meshash[HMAC_LEN], sigcopy[PUBKEY_LEN];
     unsigned int meshashlen;
     BCRYPT_PKCS1_PADDING_INFO padding;
 
@@ -881,7 +871,8 @@ int verify_RSA_sig(RSA_key_t rsa, int hashtype,
     }
 
     padding.pszAlgId = get_hash(hashtype);
-    sstatus = NCryptVerifySignature(rsa, &padding, meshash, meshashlen, sig,
+    memcpy(sigcopy, sig, siglen);
+    sstatus = NCryptVerifySignature(rsa, &padding, meshash, meshashlen, sigcopy,
                                     siglen, BCRYPT_PAD_PKCS1);
     if (!BCRYPT_SUCCESS(sstatus)) {
         mserror("NCryptVerifySignature failed", sstatus);
@@ -974,24 +965,35 @@ int verify_ECDSA_sig(EC_key_t ec, int hashtype,
  * Creates an ECDH key based on two EC keys, one public and one private
  */
 int get_ECDH_key(EC_key_t pubkey, EC_key_t privkey, unsigned char *key,
-                 unsigned int *keylen)
+                 unsigned int *keylen, int kdf_hash)
 {
     SECURITY_STATUS sstatus;
     NCRYPT_SECRET_HANDLE secret;
     int _len;
+    NCryptBuffer ncbuff;
+    NCryptBufferDesc ncbuffdesc;
+    LPCWSTR hash_str;
 
     sstatus = NCryptSecretAgreement(privkey, pubkey, &secret, 0);
     if (!BCRYPT_SUCCESS(sstatus)) {
         mserror("NCryptSecretAgreement failed", sstatus);
         return 0;
     }
-    sstatus = NCryptDeriveKey(secret, BCRYPT_KDF_HASH, NULL, NULL, 0, &_len, 0);
+    hash_str = get_hash(kdf_hash);
+    ncbuff.cbBuffer = (LONG)(wcslen(hash_str) + 1) * 2;
+    ncbuff.BufferType =  KDF_HASH_ALGORITHM;
+    ncbuff.pvBuffer = (LPWSTR)hash_str;
+    ncbuffdesc.ulVersion =  NCRYPTBUFFER_VERSION;
+    ncbuffdesc.cBuffers = 1;
+    ncbuffdesc.pBuffers = &ncbuff;
+    sstatus = NCryptDeriveKey(secret, BCRYPT_KDF_HASH, &ncbuffdesc,
+                              NULL, 0, &_len, 0);
     if (!BCRYPT_SUCCESS(sstatus)) {
         mserror("NCryptDeriveKey failed", sstatus);
         return 0;
     }
-    sstatus = NCryptDeriveKey(secret, BCRYPT_KDF_HASH, NULL, key, _len,
-                              keylen, 0 );
+    sstatus = NCryptDeriveKey(secret, BCRYPT_KDF_HASH, &ncbuffdesc, 
+                              key, _len, keylen, 0 );
     if (!BCRYPT_SUCCESS(sstatus)) {
         mserror("NCryptDeriveKey failed", sstatus);
         return 0;
@@ -1410,7 +1412,7 @@ void free_RSA_key(RSA_key_t rsa)
 {
     SECURITY_STATUS sstatus;
 
-    if (!BCRYPT_SUCCESS(sstatus = NCryptFreeObject(rsa))) {
+    if (rsa && !BCRYPT_SUCCESS(sstatus = NCryptFreeObject(rsa))) {
         mserror("NCryptFreeObject failed", sstatus);
     }
 }
@@ -1419,7 +1421,7 @@ void free_EC_key(EC_key_t ec)
 {
     SECURITY_STATUS sstatus;
 
-    if (!BCRYPT_SUCCESS(sstatus = NCryptFreeObject(ec))) {
+    if (ec && !BCRYPT_SUCCESS(sstatus = NCryptFreeObject(ec))) {
         mserror("NCryptFreeObject failed", sstatus);
     }
 }
