@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2023   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -757,6 +757,8 @@ int verify_ECDSA_sig(EC_key_t ec, int hashtype,
         return 0;
     }
 #else
+    BN_free(_sig->r);
+    BN_free(_sig->s);
     _sig->r = r;
     _sig->s = s;
 #endif
