@@ -386,9 +386,9 @@ void check_pending(struct pr_group_list_t *group, int hostidx,
         if (pending->count == 1) {
             gettimeofday(&pending->rx_tstamp, NULL);
             pending->tstamp = group->destinfo[hostidx].regtime;
-            glog3(group, "send time = %d.%06d",
+            glog3(group, "send time = %ld.%06ld",
                          pending->tstamp.tv_sec, pending->tstamp.tv_usec);
-            glog3(group, "rx time = %d.%06d",
+            glog3(group, "rx time = %ld.%06ld",
                          pending->rx_tstamp.tv_sec, pending->rx_tstamp.tv_usec);
         }
         break;
@@ -405,9 +405,9 @@ void check_pending(struct pr_group_list_t *group, int hostidx,
                 tstamp_usec |= ntohl(fileinfoack->tstamp_lo);
                 pending->tstamp = usec_to_tv(tstamp_usec);
             }
-            glog3(group, "send time = %d.%06d",
+            glog3(group, "send time = %ld.%06ld",
                          pending->tstamp.tv_sec, pending->tstamp.tv_usec);
-            glog3(group, "rx time = %d.%06d",
+            glog3(group, "rx time = %ld.%06ld",
                          pending->rx_tstamp.tv_sec, pending->rx_tstamp.tv_usec);
         }
         pending->file_id = ntohs(fileinfoack->file_id);
