@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2019   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ SOCKET listener;
 char pidfile[MAXPATHNAME];
 char keyfile[MAXLIST][MAXPATHNAME], keyinfo[MAXLIST][MAXPATHNAME];
 int proxy_type, debug, rcvbuf, dscp, keyfile_count, keyinfo_count;
-int hb_interval, priority;
+int hb_interval, priority, user_abort, use_ssm;
 unsigned int ttl;
 char portname[PORTNAME_LEN], out_portname[PORTNAME_LEN];
 int port, out_port;
@@ -77,8 +77,8 @@ int keyfile_count, key_count, pub_multi_count, interface_count, sys_keys;
 struct iflist out_if;
 union key_t privkey[MAXLIST];
 int privkey_type[MAXLIST];
-union key_t dhkey;
-uint8_t ecdh_curve;
+union key_t v4_dhkey;
+uint8_t v4_ecdh_curve;
 struct pr_group_list_t group_list[MAXLIST];
 
 extern char *optarg;
@@ -126,10 +126,6 @@ void add_hosts_by_name(struct fp_list_t *list, int *list_count,
         } else {
             fingerprint = NULL;
         }
-        if (strlen(hostid) >= DESTNAME_LEN) {
-            fprintf(stderr, "Server/Client list %s: name too long\n", filename);
-            exit(ERR_PARAM);
-        }
 
         remote_uid = strtoul(hostid, NULL, 16);
         if ((remote_uid == 0xffffffff) || (remote_uid == 0)) {
@@ -198,9 +194,11 @@ void set_defaults(void)
     pub_multi_count = 0;
     sys_keys = 0;
     priority = 0;
-    ecdh_curve = 0;
+    v4_ecdh_curve = DEF_CURVE;
     max_log_size = 0;
     max_log_count = DEF_MAX_LOG_COUNT;
+    user_abort = 0;
+    use_ssm = 0;
 }
 
 /**
@@ -212,7 +210,7 @@ void process_args(int argc, char *argv[])
     int c, i, listidx, rval;
     long tmpval;
     char *p, *p2, *hoststr, *portstr, pubname[INET6_ADDRSTRLEN];
-    const char opts[] = "s:crdx:p:t:Q:N:O:U:q:mh:H:g:n:B:L:P:C:S:e:k:K:I:M:";
+    const char opts[] = "s:crdx:p:t:Q:N:O:U:q:mh:H:g:n:B:L:P:C:oS:e:k:K:I:M:";
 
     set_defaults();
     srand((unsigned int)time(NULL) ^ getpid());
@@ -358,7 +356,7 @@ void process_args(int argc, char *argv[])
             while (p != NULL) {
                 p2 = strchr(p, ':');
                 if (p2) {
-                    hoststr = strdup(p);
+                    hoststr = p;
                     hoststr[p2 - p] = '\x0';
                     portstr = p2 + 1;
                 } else {
@@ -378,6 +376,7 @@ void process_args(int argc, char *argv[])
                 }
                 memcpy(&hb_hosts[hbhost_count++], ai_rval->ai_addr,
                         ai_rval->ai_addrlen);
+                freeaddrinfo(ai_rval);
                 p = strtok(NULL, ",");
             }
             break;
@@ -421,12 +420,15 @@ void process_args(int argc, char *argv[])
         case 'C':
             add_hosts_by_name(client_fp, &client_fp_count, optarg, 0);
             break;
+        case 'o':
+            use_ssm = 1;
+            break;
         case 'S':
             add_hosts_by_name(server_fp, &server_fp_count, optarg, 1);
             break;
         case 'e':
-            ecdh_curve = get_curve(optarg);
-            if (ecdh_curve == 0) {
+            v4_ecdh_curve = get_curve(optarg);
+            if (v4_ecdh_curve == 0) {
                 fprintf(stderr, "Invalid curve\n");
                 exit(ERR_PARAM);
             }
@@ -521,7 +523,7 @@ void process_args(int argc, char *argv[])
         }
     }
     if (proxy_type != CLIENT_PROXY) {
-        if (server_fp_count) {
+        if (use_ssm) {
             for (i = 0; i < pub_multi_count; i++) {
                 if (!is_multicast(&pub_multi[i], 1)) {
                     if ((rval = getnameinfo((struct sockaddr *)&pub_multi[i],
