@@ -191,7 +191,7 @@ void mainloop(void)
         }
         tv = getrecenttimeout();
         if (tv) {
-            log5(0, 0, 0, "timeout: %d.%06d", tv->tv_sec, tv->tv_usec);
+            log5(0, 0, 0, "timeout: %ld.%06ld", tv->tv_sec, tv->tv_usec);
         }
         if (read_packet(listener, &src, buf, &packetlen,
                         MAXMTU, tv, &tos) <= 0) {
