@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2022   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -145,7 +145,7 @@ int send_announce(const struct finfo_t *finfo, int attempt, int open)
     encinfo = (struct enc_info_he *)(privateaddr + iplen);
 
     set_uftp_header(header, ANNOUNCE, finfo->group_id, finfo->group_inst,
-                    grtt, destcount);
+                    get_adv_grtt(grtt), destcount);
     announce->func = ANNOUNCE;
     if (sync_mode) {
         announce->flags |= FLAG_SYNC_MODE;
@@ -234,7 +234,7 @@ int send_regconf(const struct finfo_t *finfo, int attempt, int do_regconf)
     regconf = (struct regconf_h *)(buf + sizeof(struct uftp_h));
 
     set_uftp_header(header, REG_CONF, finfo->group_id, finfo->group_inst,
-                    grtt, destcount);
+                    get_adv_grtt(grtt), destcount);
     regconf->func = REG_CONF;
     regconf->hlen = sizeof(struct regconf_h) / 4;
 
@@ -275,7 +275,7 @@ int send_keyinfo(const struct finfo_t *finfo, int attempt)
     }
 
     set_uftp_header(header, KEYINFO, finfo->group_id, finfo->group_inst,
-                    grtt, destcount);
+                    get_adv_grtt(grtt), destcount);
     keyinfo->func = KEYINFO;
     keyinfo->hlen = (uint8_t)((sizeof(struct keyinfo_h) + siglen) / 4);
     keyinfo->siglen = htons(siglen);
@@ -409,7 +409,7 @@ int send_fileinfo(const struct finfo_t *finfo, int attempt)
     filename = (char *)fileinfo + sizeof(struct fileinfo_h);
 
     set_uftp_header(header, FILEINFO, finfo->group_id, finfo->group_inst,
-                    grtt, destcount);
+                    get_adv_grtt(grtt), destcount);
     fileinfo->func = FILEINFO;
     fileinfo->ftype = finfo->ftype;
     fileinfo->file_id = htons(finfo->file_id);
