@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2022   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -81,7 +81,7 @@ void send_abort(const struct finfo_t *finfo, const char *message,
     abort_hdr = (struct abort_h *)(buf + sizeof(struct uftp_h));
 
     set_uftp_header(header, ABORT, finfo->group_id, finfo->group_inst,
-                    grtt, destcount);
+                    get_adv_grtt(grtt), destcount);
     header->seq = htons(send_seq++);
     abort_hdr->func = ABORT;
     abort_hdr->hlen = sizeof(struct abort_h) / 4;
@@ -436,3 +436,16 @@ int recalculate_grtt(const struct finfo_t *finfo, int grtt_set,
         return 0;
     }
 }
+
+/**
+ * Returns the advertised GRTT based on the given value
+ * Enforces a floor of the interpacket transmission time
+ */
+double get_adv_grtt(double l_grtt)
+{
+    if (l_grtt < packet_wait / 1000000000.0) {
+        return packet_wait / 1000000000.0;
+    } else {
+        return l_grtt;
+    }
+}
