@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2019   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -74,7 +74,6 @@ char basedir[MAXDIR][MAXDIRNAME], destfname[MAXPATHNAME];
 char statusfilename[MAXPATHNAME];
 FILE *status_file;
 struct iflist ifl[MAX_INTERFACES];
-int keytype, hashtype, sigtype, keyextype, newkeylen, sys_keys;
 int blocksize, datapacketsize;
 int ifl_len, destcount, filecount, excludecount, basedircount;
 struct iflist out_if;
@@ -90,9 +89,10 @@ uint32_t server_id;
  */
 union key_t privkey, dhkey;
 unsigned char rand1[RAND_LEN], groupmaster[MASTER_LEN];
-uint8_t groupsalt[MAXIV], groupkey[MAXKEY], grouphmackey[HMAC_LEN];
+uint8_t server_app_iv[MAXIV], server_app_key[MAXKEY], *servercontext;
 uint64_t ivctr;
-int ivlen, keylen, hmaclen, privkeylen;
+int ivlen, keylen, hashlen, servercontext_len;
+int keytype, hashtype, newkeylen, sys_keys, privkeytype;
 uint8_t ecdh_curve, ecdsa_curve;
 
 extern char *optarg;
@@ -173,8 +173,6 @@ void set_defaults(void)
     restart_groupinst = 0;
     keytype = DEF_KEYTYPE;
     hashtype = DEF_HASHTYPE;
-    sigtype = DEF_SIGTYPE;
-    keyextype = DEF_KEYEXTYPE;
     ecdh_curve = DEF_CURVE;
     strncpy(pub_multi, DEF_PUB_MULTI, sizeof(pub_multi)-1);
     pub_multi[sizeof(pub_multi)-1] = '\x0';
@@ -189,11 +187,11 @@ void set_defaults(void)
     excludecount = 0;
     basedircount = 0;
     newkeylen = 0;
-    ecdh_curve = 0;
     ecdsa_curve = 0;
     follow_links = 0;
     showtime = 0;
     sys_keys = 0;
+    privkeytype = 0;
     sync_mode = 0;
     sync_preview = 0;
     dest_is_dir = 0;
@@ -282,14 +280,6 @@ static int get_keytype(const char *name)
 {
     if (!strcmp(name, "none")) {
         return KEY_NONE;
-    } else if (!strcmp(name, "des")) {
-        return KEY_DES;
-    } else if (!strcmp(name, "3des")) {
-        return KEY_DES_EDE3;
-    } else if (!strcmp(name, "aes128-cbc")) {
-        return KEY_AES128_CBC;
-    } else if (!strcmp(name, "aes256-cbc")) {
-        return KEY_AES256_CBC;
     } else if (!strcmp(name, "aes128-gcm")) {
         return KEY_AES128_GCM;
     } else if (!strcmp(name, "aes256-gcm")) {
@@ -309,9 +299,7 @@ static int get_keytype(const char *name)
  */
 static int get_hashtype(const char *name)
 {
-    if (!strcmp(name, "sha1")) {
-        return HASH_SHA1;
-    } else if (!strcmp(name, "sha256")) {
+    if (!strcmp(name, "sha256")) {
         return HASH_SHA256;
     } else if (!strcmp(name, "sha384")) {
         return HASH_SHA384;
@@ -330,14 +318,12 @@ void process_args(int argc, char *argv[])
     int c, i, listidx, read_restart, rval;
     int64_t tmpval;
     char line[1000], *dest, *destname, filename[MAXPATHNAME], *fingerprint, *p;
-    char keylenstr[50];
     struct addrinfo ai_hints, *ai_rval;
     FILE *destfile, *excludefile, *listfile;
-    const char opts[] = "x:R:L:B:g:n:m:Y:h:w:e:ck:K:lTb:t:Q:"
+    const char opts[] = "x:R:L:B:g:n:m:Y:h:e:ck:K:lTb:t:Q:"
                         "zZI:p:u:j:qfyU:a:H:F:X:M:P:C:D:oE:S:r:s:i:W:N:";
 
     set_defaults();
-    memset(keylenstr, 0, sizeof(keylenstr));
     read_restart = 0;
 
     // read lettered arguments
@@ -414,45 +400,12 @@ void process_args(int argc, char *argv[])
                 exit(ERR_PARAM);
             }
             break;
-        case 'w':
-            if (!strcmp(optarg, "hmac")) {
-                sigtype = SIG_HMAC;
-            } else if (!strcmp(optarg, "keyex")) {
-                sigtype = SIG_KEYEX;
-            } else {
-                fprintf(stderr, "Invalid sigtype\n");
-                exit(ERR_PARAM);
-            }
-            break;
         case 'e':
-            p = strtok(optarg, ":");
-            if (!p) {
-                fprintf(stderr, "Error reading keyextype\n");
+            ecdh_curve = get_curve(optarg);
+            if (ecdh_curve == 0) {
+                fprintf(stderr, "Invalid curve\n");
                 exit(ERR_PARAM);
             }
-            if (!strcmp(p, "rsa")) {
-                keyextype = KEYEX_RSA;
-            } else if (!strcmp(p, "ecdh_rsa")) {
-                keyextype = KEYEX_ECDH_RSA;
-            } else if (!strcmp(p, "ecdh_ecdsa")) {
-                keyextype = KEYEX_ECDH_ECDSA;
-            } else {
-                fprintf(stderr, "Invalid keyextype\n");
-                exit(ERR_PARAM);
-            }
-            if ((keyextype == KEYEX_ECDH_RSA) ||
-                    (keyextype == KEYEX_ECDH_ECDSA)) {
-                p = strtok(NULL, ":");
-                if (p) {
-                    ecdh_curve = get_curve(p);
-                    if (ecdh_curve == 0) {
-                        fprintf(stderr, "Invalid curve\n");
-                        exit(ERR_PARAM);
-                    }
-                } else {
-                    ecdh_curve = DEF_CURVE;
-                }
-            }
             break;
         case 'c':
             client_auth = 1;
@@ -462,8 +415,26 @@ void process_args(int argc, char *argv[])
             keyfile[sizeof(keyfile)-1] = '\x0';
             break;
         case 'K':
-            strncpy(keylenstr, optarg, sizeof(keylenstr)-1);
-            keylenstr[sizeof(keylenstr)-1] = '\x0';
+            p = strtok(optarg, ":");
+            if (!p) {
+                fprintf(stderr, "Error reading new key\n");
+                exit(ERR_PARAM);
+            }
+            if (!strcmp("ec", p)) {
+                p = strtok(NULL, ":");
+                ecdsa_curve = get_curve(p);
+                if (ecdsa_curve == 0) {
+                    fprintf(stderr, "Invalid curve\n");
+                    exit(ERR_PARAM);
+                }
+            } else if (!strcmp("rsa", p)) {
+                p = strtok(NULL, ":");
+                newkeylen = atoi(p);
+                if ((newkeylen < RSA_MIN) || (newkeylen > RSA_MAX)) {
+                    fprintf(stderr, "Invalid new key length\n");
+                    exit(ERR_PARAM);
+                }
+            }
             break;
         case 'l':
             follow_links = 1;
@@ -879,30 +850,6 @@ void process_args(int argc, char *argv[])
         exit(ERR_PARAM);
     }
 
-    if (keytype == KEY_NONE) {
-        hashtype = HASH_NONE;
-        sigtype = SIG_NONE;
-        keyextype = KEYEX_NONE;
-    }
-    if (is_auth_enc(keytype)) {
-        sigtype = SIG_AUTHENC;
-    }
-    if (strcmp(keylenstr, "")) {
-        if (keyextype == KEYEX_ECDH_ECDSA) {
-            ecdsa_curve = get_curve(keylenstr);
-            if (ecdsa_curve == 0) {
-                fprintf(stderr, "Invalid curve\n");
-                exit(ERR_PARAM);
-            }
-        } else if ((keyextype == KEYEX_RSA) || (keyextype == KEYEX_ECDH_RSA)) {
-            newkeylen = atoi(keylenstr);
-            if ((newkeylen < 512) || (newkeylen > 2048)) {
-                fprintf(stderr, "Invalid new key length\n");
-                exit(ERR_PARAM);
-            }
-        }
-    }
-
     if (filecount != 0) {
         if (argc > 0) {
             fprintf(stderr, "Warning: ignoring paths "
