@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2022   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -483,7 +483,6 @@ void key_init(void)
  */
 void initialize(void)
 {
-    atexit(cleanup);
     init_log_mux = 1;
     init_log(0);
 
@@ -515,6 +514,9 @@ void initialize(void)
     }
 
     key_init();
+    // atexit call MUST be after OpenSSL init, which has its own 
+    // handler function to ensure proper order of cleanup
+    atexit(cleanup);
     create_sockets();
 
     // Size of data packet, used in transmission speed calculations
@@ -544,8 +546,13 @@ void initialize(void)
             packet_wait = (int64_t)(1000000000.0 * datapacketsize / rate);
         }
     } else if (cc_type == CC_TFMCC) {
-        // Initialize the rate to the default rate for control message timing
-        packet_wait = (int64_t)(1000000000.0 * datapacketsize / 128000);
+        int64_t ann_rate = DEF_RATE;
+        if ((init_rate > 0) && (init_rate < ann_rate)) {
+            ann_rate = init_rate;
+        } else if ((max_rate > 0) && (max_rate < ann_rate)) {
+            ann_rate = max_rate;
+        }
+        packet_wait = (int64_t)(1000000000.0 * datapacketsize / ann_rate);
     }
 }
 
