@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2023   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -80,6 +80,12 @@ int handle_announce_phase(unsigned char *packet, unsigned char *decrypted,
                          ntohl(header->src_id));
             return 0;
         }
+        if (!destlist[hostidx].encinfo)
+        {
+            glog3(finfo, "Got encrypted packet from unregistered receiver %s",
+                         destlist[hostidx].name);
+            return 0;
+        }
         // first try c_hs_key, then c_app_key
         func = NULL;
         log_sav = log_level;
@@ -401,6 +407,7 @@ int announce_phase(struct finfo_t *finfo)
                 fprintf(status_file, "CONNECT;failed;%s\n", destlist[i].name);
             }
         }
+        fflush(status_file);
     }
     free(packet);
     free(decrypted);
@@ -634,11 +641,11 @@ THREAD_FUNC transfer_send_thread(void *infop)
     }
     data = (unsigned char *)fileseg + (fileseg->hlen * 4);
     set_uftp_header(header, FILESEG, finfo->group_id, finfo->group_inst,
-                    l_adv_grtt, destcount);
+                    get_adv_grtt(l_adv_grtt), destcount);
 
     gettimeofday(&start_time, NULL);
     gettimeofday(&last_sent, NULL);
-    overage = 0;
+    overage = l_packet_wait;  // Don't wait before sending first packet
     offset = 0;
 
     lseek_func(file, 0, SEEK_SET);
