@@ -1,7 +1,7 @@
 /*
  *  UFTP - UDP based FTP with multicast
  *
- *  Copyright (C) 2001-2020   Dennis A. Bush, Jr.   bush@tcnj.edu
+ *  Copyright (C) 2001-2022   Dennis A. Bush, Jr.   bush@tcnj.edu
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ int send_doneconf(const struct finfo_t *finfo, int attempt)
     doneconf = (struct doneconf_h *)(buf + sizeof(struct uftp_h));
 
     set_uftp_header(header, DONE_CONF, finfo->group_id, finfo->group_inst,
-                    grtt, destcount);
+                    get_adv_grtt(grtt), destcount);
     doneconf->func = DONE_CONF;
     doneconf->hlen = sizeof(struct doneconf_h) / 4;
 
@@ -101,7 +101,7 @@ int send_done(const struct finfo_t *finfo, int attempt, int section,
     done = (struct done_h *)(buf + sizeof(struct uftp_h));
 
     set_uftp_header(header, DONE, finfo->group_id, finfo->group_inst,
-                    l_grtt, destcount);
+                    get_adv_grtt(l_grtt), destcount);
     done->func = DONE;
     done->hlen = sizeof(struct done_h) / 4;
     done->file_id = htons(finfo->file_id);
@@ -188,7 +188,7 @@ void send_cong_ctrl(const struct finfo_t *finfo, double l_grtt,
     cong_ctrl = (struct cong_ctrl_h *)(buf + sizeof(struct uftp_h));
 
     set_uftp_header(header, CONG_CTRL, finfo->group_id, finfo->group_inst,
-                    l_grtt, destcount);
+                    get_adv_grtt(l_grtt), destcount);
     header->seq = htons(send_seq++);
     cong_ctrl->func = CONG_CTRL;
     cong_ctrl->hlen = sizeof(struct cong_ctrl_h) / 4;
@@ -378,7 +378,11 @@ void handle_tfmcc_ack_info(const struct finfo_t *finfo,
 
     // TODO: should we be checking the advertised GRTT instead of the real GRTT?
     if (destlist[hostidx].rtt > grtt) {
-        grtt = destlist[hostidx].rtt;
+        if (destlist[hostidx].rtt < max_grtt) {
+            grtt = destlist[hostidx].rtt;
+        } else {
+            grtt = max_grtt;
+        }
     }
     if (!flag_ss) {
         slowstart = 0;
@@ -703,6 +707,7 @@ void print_status_file(const struct finfo_t *finfo, struct timeval start_time)
                     destlist[i].total_time,
                     throughput);
         }
+        fflush(status_file);
         return;
     }
 
@@ -770,6 +775,7 @@ void print_status_file(const struct finfo_t *finfo, struct timeval start_time)
             break;
         }
     }
+    fflush(status_file);
 }
 
 /**
