@@ -61,6 +61,10 @@ extern mux_t log_mux;
 void init_log(int _debug);
 void close_log(void);
 void roll_log(void);
+
+#ifdef __GNUC__
+__attribute__ ((format (printf, 9, 10)))
+#endif
 void logfunc(uint32_t group_id, uint8_t group_inst, uint16_t file_id,
              int level, int _showtime, int newline, int err, int sockerr,
              const char *str, ...);
